


float4x4 matWorldViewProj	: WORLDVIEWPROJ;	//our world view projection matrix
float4x4 matWorldI			: WORLDVIEWIT;		//our inverse transpose matrix
float4x4 matWorld			: WORLD;			//our world matrix
float4 lightDir				= float4(0,0,1,0);	//our light position
float3 planeDir				= float3(1,0,0);
float3 pointOnPlane         = float3(0,0,5);

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  ooooooooo  ooooooo  oooo  oooo  ooooooooo  oooo  oooo  ooooooo   ooooooo  //
//  ooooooooo  ooo       ooo  ooo   ooooooooo  oooo  oooo  ooo  ooo  ooo      //
//     ooo     ooooooo    oo  oo       ooo     oooo  oooo  ooo  ooo  ooooooo  //
//     ooo     ooooooo     oooo        ooo     oooo  oooo  ooo ooo   ooooooo  //
//     ooo     ooo        oooooo       ooo      ooo  ooo   ooooooo   ooo      //
//     ooo     ooooooo  oooo  oooo     ooo       oooooo    ooo  ooo  ooooooo  //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

texture texture0;

 
sampler2D texSampler0 : TEXUNIT0 = sampler_state
{
	Texture	  = (texture0);
    MIPFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
};

 
 
 
//application to vertex structure
struct vin
{ 
    float4 position   : POSITION0;
    float3 normal	  : NORMAL;
    float2 tex0       : TEXCOORD0;
};
 
//vertex to pixel shader structure
struct vout
{
    float4 position	   : POSITION0;
    float4 colour      : COLOR0;
    float2 tex0        : TEXCOORD0;
    float3 tex1        : TEXCOORD1;
};
 
 

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//   ooo       ooo  oooooooo  oooooooo   ooooooooo  oooooooo  ooo     ooo     //
//    ooo     ooo   ooo       ooo   ooo  ooooooooo  ooo        ooo   ooo      //
//     ooo   ooo    ooo       ooo   ooo     ooo     ooo         ooooooo       //
//      ooo ooo     oooooooo  ooo   ooo     ooo     oooooooo     ooooo        //
//       ooooo      ooo       oooooooo      ooo     ooo         ooooooo       //
//        ooo       ooo       ooo  oo       ooo     ooo        ooo   ooo      //
//         o        oooooooo  ooo   ooo     ooo     oooooooo  ooo     ooo     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

float4 DiffuseLighting(float4 vInColour, float3 vNormal)
{

	vNormal.z = abs(vNormal.z);
	/*
	if(vNormal.z < -0.2 )
	{
		vNormal.z = -vNormal.z;
		vInColour = float4(0,0,1,1);
	}
	*/
	
	vNormal = normalize(vNormal);
	
	float4 ambient = {0.1, 0.1, 0.1, 0.1};
	ambient = ambient * vInColour;
	
	float4 diffuse = vInColour;
	float4 light = normalize( lightDir );
	vInColour = ambient + diffuse * dot(vNormal, light);
	vInColour.a = 1.0f;
	return vInColour;	
}


//VERTEX SHADER
void vs( in vin IN, out vout OUT )
{
    //getting to position to object space
    OUT.position    = mul(IN.position, matWorldViewProj);
	
    //getting the position of the vertex in the world
    float3 posWorld = mul(IN.position, matWorld);
    
	//transform our normal vector so its facing the right way!
	float3 normal   = mul( matWorldI, IN.normal );
	
	//pass along some colour information
	OUT.colour = DiffuseLighting(float4(1,1,1,1), normal);
	
	// So our tex1 values are interpolation across the triangles surface
	OUT.tex1 = posWorld;

	//pass texture data on to pixel stage
	OUT.tex0 = IN.tex0;
}

 
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//            oooooooo  ooo   oooo     oooo    oooooo    ooo                  //
//            oo    oo  ooo    oooo   oooo     oooooo    ooo                  //
//            oo    oo  ooo      ooo ooo       ooo       ooo                  //
//            oooooo    ooo       ooooo        oooooo    ooo                  //
//            oo        ooo      ooo ooo       ooo       ooo                  //
//            oo        ooo     oooo  oooo     oooooo    ooooooooo            //
//            oo        ooo    oooo    oooo    oooooo    ooooooooo            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

//PIXEL SHADER
float4 ps( in vout IN ) : COLOR
{
    //get our colour
    float4 finalcolor = float4(0.5, 0.2, 0.2, 1.0f) * IN.colour;

    float3 plane_normal = normalize(planeDir);
    
    float d = dot(pointOnPlane, plane_normal);
    
    float3 pos = IN.tex1;
    
	float k = dot(pos, plane_normal)-d;
	
	if( k > 0 ) finalcolor.a = 0;

    //return final pixel colour
	return finalcolor;
}


float4 ps1( in vout IN ) : COLOR
{
    //get our colour
    float4 finalcolor = IN.colour;

    float3 plane_normal = normalize(planeDir);
    
    float d = dot(pointOnPlane, plane_normal);
    
    float3 pos = IN.tex1;
    
	float k = dot(pos, plane_normal)-d;
	
	if( k > 0 ) finalcolor.a = 0;

    //return final pixel colour
	return finalcolor;
}
 
float4 ps2( in vout IN ) : COLOR
{
    //get our colour
    float4 finalcolor = IN.colour;
	finalcolor.a = 0.2f;

    float3 plane_normal = normalize(planeDir);
    
    float d = dot(pointOnPlane, plane_normal);
    
    float3 pos = IN.tex1;
    
	float k = dot(pos, plane_normal)-d;
	
	if( k < 0 ) finalcolor.a = 0;

    //return final pixel colour
	return finalcolor;
}

technique Effect
{
	// draws inside of the shape
    pass p0
    {
        vertexshader = compile vs_1_1 vs();
        pixelshader  = compile ps_2_0 ps();
        
        AlphaBlendEnable = true;
		SrcBlend = srcalpha;
		DestBlend = invsrcalpha;
		
		ZEnable=false;
		ZWriteEnable = false;
		CullMode = ccw;
    }
    
    // outside of the +ve side of the plane/slice
    pass p1
    {
        vertexshader = compile vs_1_1 vs();
        pixelshader  = compile ps_2_0 ps1();
        
        AlphaBlendEnable = true;
		SrcBlend = srcalpha;
		DestBlend = invsrcalpha;
		
		ZEnable=true;
		ZWriteEnable = true;
		CullMode = cw;
    }
    
    // draws the -ve side of the plane/slice (wireframe)
    pass p2
    {
        vertexshader = compile vs_1_1 vs();
        pixelshader  = compile ps_2_0 ps2();
        
        AlphaBlendEnable = true;
		SrcBlend = srcalpha;
		DestBlend = invsrcalpha;
		
		ZEnable=true;
		ZWriteEnable = true;
		CullMode = cw;
    }
    
}


